function [ax,ay] = peakHilbertAF(w,sr,slen,plen,ppeak,emin,emax)
% This function takes a waveform, extracts peak events from its 
% Hilbert envelope, and then runs Allan Factor analysis on the 
% peak event series. The input arguments are as follows:
% - w is the waveform Nx1 vector. Downsampling should be done beforehand.
% - sr is the number of waveform samples per second.
% - slen is the number of samples to process per analysis. The code will
%   step through w in slen segments, and average the results across segments.
%   The segment is end-aligned, and will overlap with the previous segment
%   if slen does not divide evenly into length(w). 
% - plen is the minimum interval between two peaks, in samples. 
% - ppeak is the desired proportion of peaks, used to determine the peak
%   amplitude threshold.
% - emin and emax set the max and min timescales, respectively. w is
%   partitioned into 1/2^emin windows for the longest timescale analyzed, 
%   and 1/2^emax windows for the shortest timescale

% the default segment length is the entire waveform
if slen == 0
    slen = length(w)
end

N = ceil(length(w)/slen);
for i=1:N
    if i==N
        cw = w(end-slen+1:end);
    else
        cw = w((i-1)*slen+1:i*slen);
    end
    
    % compute Hilbert envelope of waveform
    disp(sprintf('compute envelope %d of %d...',i,N));
    env = abs(hilbert(cw));
    
    % loop through the envelope to find and preserve peaks separated by
    % +/- plen, which serves as lowpass filter to limit the rate of peaks 
    disp('find peaks...');
    n = slen-plen+1;
    for j = 1:n
        [y,idx] = max(env(j:j+plen-1));
        env(j:j+plen-1)=0;
        env(j+idx-1)=y;
    end

    % ppeak is the mean proportion of peaks in each segment.  
    % ppeak is used to set threshold cmax for choosing largest peaks.
    envs = sort(env(find(env>0)),'descend');
    cmax = envs(floor(slen*ppeak)); pp = env>cmax; 
    % run Allan Factor variance (aka normalized wavelet variance) 
    %   for each segment and average. 
    %   ax are window sizes T in seconds, assuming sr is in Hertz
    %   ay are AF variances for each T in ax
    disp('run Allan Factor...');
    [ay(:,i),ax(:,i)]=AFanalysis(pp,emin,emax,1/sr);
end
if N > 1
    ax=mean(ax'); ay=mean(ay'); 
end
end